@@ -1,27 +1,41 @@
-# -*- coding: utf-8 -*-
-from __future__ import absolute_import, division, print_function
-__authors__ = ["Alex Hamilton, Stuart Mumford"]
-__email__ = "stuart@mumford.me.uk"
-
-from sunpy.util.metadata import MetaDict
-import itertools
+"""
+This module provies metadata support for `~sunpy.timeseries.TimeSeries`.
+"""
 import copy
-
 import warnings
-import inspect
+import itertools
 
 from sunpy.time import TimeRange, parse_time
+from sunpy.time.time import _variables_for_parse_time_docstring
+from sunpy.util.decorators import add_common_docstring
+from sunpy.util.exceptions import SunpyUserWarning
+from sunpy.util.metadata import MetaDict
 
-class TimeSeriesMetaData:
+__all__ = ["TimeSeriesMetaData"]
+
+
+class TimeSeriesMetaData(object):
     """
-    An object used to store metadata for TimeSeries objects that enables multiple
-    TimeSeries metadata to be concatenated in an organised fashion.
+    Used to store metadata for `~sunpy.timeseries.TimeSeries` that enables
+    multiple `sunpy.timeseries.TimeSeries` metadata to be concatenated in an
+    organized fashion.
+
+    Parameters
+    ----------
+    meta : `dict`, `MetaDict`, `tuple`, `list`, optional
+        The metadata giving details about the time series data/instrument.
+        Defaults to `None`.
+    timerange : `~sunpy.time.TimeRange`
+        A `~sunpy.time.TimeRange` representing the timespan of the data. Defaults to `None`.
+    colnames : `list`, optional
+        A mapping from column names in ``data`` to the physical units of that column.
+        Defaults to `None`.
 
     Attributes
     ----------
     metadata : `list` of `tuple`
         The list of 3-tuples which each represent a source files metadata.
-        The tuples consist of: ( TimeRange, [ colnames ], MetaDict(metadata) )
+        The tuples consist of: ``(TimeRange, [colnames], MetaDict(metadata))``.
 
     Examples
     --------
@@ -29,17 +43,39 @@ class TimeSeriesMetaData:
     >>> from sunpy.time import TimeRange, parse_time
     >>> from sunpy.util import MetaDict
     >>> tr = TimeRange('2012-06-01 00:00','2012-06-02 00:00')
-    >>> md = TimeSeriesMetaData(timerange=tr, colnames=['GOES'], meta=MetaDict([('goes_key','goes_val')]))
+    >>> md = TimeSeriesMetaData(timerange=tr, colnames=['GOES'],
+    ...                         meta=MetaDict([('goes_key','goes_val')]))
     >>> tr2 = TimeRange('2012-06-01 12:00','2012-06-02 12:00')
     >>> md.append(tr2, ['EVE'], MetaDict([('eve_key','eve_val')]))
     >>> md.find(parse_time('2012-06-01T21:08:12'))
+    |-------------------------------------------------------------------------------------------------|
+    |TimeRange                  | Columns         | Meta                                              |
+    |-------------------------------------------------------------------------------------------------|
+    |2012-06-01T00:00:00.000    | GOES            | goes_key: goes_val                                |
+    |            to             |                 |                                                   |
+    |2012-06-02T00:00:00.000    |                 |                                                   |
+    |-------------------------------------------------------------------------------------------------|
+    |2012-06-01T12:00:00.000    | EVE             | eve_key: eve_val                                  |
+    |            to             |                 |                                                   |
+    |2012-06-02T12:00:00.000    |                 |                                                   |
+    |-------------------------------------------------------------------------------------------------|
+    <BLANKLINE>
     >>> md.find(parse_time('2012-06-01T21:08:12')).columns
+    ['EVE', 'GOES']
     >>> md.find(parse_time('2012-06-01T21:08:12')).values()
+    ['eve_val', 'goes_val']
     >>> md.find(parse_time('2012-06-01T21:08:12')).metas
-    >>> md.find(parse_time('2012-06-01T21:08:12'), 'GOES')   # doctest: +SKIP
+    [MetaDict([('goes_key', 'goes_val')]), MetaDict([('eve_key', 'eve_val')])]
+    >>> md.find(parse_time('2012-06-01T21:08:12'), 'GOES')
+    |-------------------------------------------------------------------------------------------------|
+    |TimeRange                  | Columns         | Meta                                              |
+    |-------------------------------------------------------------------------------------------------|
+    |2012-06-01T00:00:00.000    | GOES            | goes_key: goes_val                                |
+    |            to             |                 |                                                   |
+    |2012-06-02T00:00:00.000    |                 |                                                   |
+    |-------------------------------------------------------------------------------------------------|
     """
-
-    def __init__(self, meta=None, timerange=None, colnames=None, **kwargs):
+    def __init__(self, meta=None, timerange=None, colnames=None):
         self.metadata = []
         # Parse in arguments
         if not isinstance(meta, type(None)):
@@ -58,28 +94,30 @@ class TimeSeriesMetaData:
                 self.metadata.append((timerange, colnames, MetaDict()))
             elif isinstance(timerange, TimeRange):
                 self.metadata.append((timerange, [], MetaDict()))
-                warnings.warn("No time range given for metadata. This will mean the metadata can't be linked to columns in data.", Warning)
+                warnings.warn("No time range given for metadata. "
+                              "This will mean the metadata can't be linked "
+                              "to columns in data.", SunpyUserWarning)
             else:
-                raise ValueError("You cannot create a TimeSeriesMetaData object without specifying a TimeRange")
+                raise ValueError("You cannot create a TimeSeriesMetaData "
+                                 "object without specifying a TimeRange")
 
     def __eq__(self, other):
         """
-        Check two TimeSeriesMetaData objects are the same, they have the same
-        entries in the same order.
+        Checks to see if two `~sunpy.timeseries.TimeSeriesMetaData` are the
+        same, they have the same entries in the same order.
 
         Parameters
         ----------
         other : `~sunpy.timeseries.metadata.TimeSeriesMetaData`
-            The second TimeSeriesMetaData object to compare with.
+            The second `~sunpy.timeseries.metadata.TimeSeriesMetaData` to compare with.
 
         Returns
         -------
-        result : `bool`
+        `bool`
         """
         match = True
         if len(self.metadata) == len(other.metadata):
-            for i in range(0,len(self.metadata)):
-                #
+            for i in range(0, len(self.metadata)):
                 if self.metadata[i] != other.metadata[i]:
                     match = False
         else:
@@ -88,39 +126,36 @@ class TimeSeriesMetaData:
 
     def __ne__(self, other):
         """
-        Check two TimeSeriesMetaData objects are not the same, they don't have
-        same entries in the same order.
+        Checks to see if two `~sunpy.timeseries.TimeSeriesMetaData` are the not
+        the same, they don't have same entries in the same order.
 
         Parameters
         ----------
-        other : `~sunpy.timeseries.GenericTimeSeries`
-            The second TimeSeries object to compare with.
+        other : `~sunpy.timeseries.TimeSeriesMetaData`
+            The second `~sunpy.timeseries.TimeSeriesMetaData` to compare with.
 
         Returns
         -------
-        result : `bool`
+        `bool`
         """
         return not self == other
 
     def append(self, timerange, columns, metadata, **kwargs):
         """
-        Add the given metadata MetaDict into the metadata list as a tuple with
-        it's TimeRange and colnames (list).
-        Will add the new entry so the list is in chronological order for the
-        TimeRange.start datetime values.
+        Add the given metadata into the current metadata.
+
+        Will add the new entry so the list is in chronological order.
 
         Parameters
         ----------
         timerange : `~sunpy.time.TimeRange`
             The timerange for which a given metadict is relevant. This will
-            generally initilly be the full range of the original file, but if
+            generally initially be the full range of the original file, but if
             the TimeSeries gets truncated this may change appropriately.
-
-        columns : `str`
-            A list of the colomn name strings that the metadata is relevant for.
-
+        columns : `list`
+            A list of the column name strings that the metadata is relevant for.
         metadata : `~sunpy.util.metadata.MetaDict` or `OrderedDict` or `dict`
-            The dictionary holding the metadata.
+            The object holding the metadata.
         """
         # Parameters
         metadata = MetaDict(metadata)
@@ -132,8 +167,7 @@ class TimeSeriesMetaData:
                 if timerange.start > meta[0].start:
                     pos = i + 1
         else:
-            raise ValueError(
-                'Incorrect datatime or data for append to TimeSeriesMetaData.')
+            raise ValueError('Incorrect datatime or data for append to TimeSeriesMetaData.')
 
         # Prepare tuple to append.
         new_metadata = (timerange, columns, metadata)
@@ -149,28 +183,26 @@ class TimeSeriesMetaData:
         if not duplicate:
             self.metadata.insert(pos, new_metadata)
 
-    def find_indices(self, time=None, colname=None, **kwargs):
+    @add_common_docstring(**_variables_for_parse_time_docstring())
+    def find_indices(self, time=None, colname=None):
         """
-        Find the indices for all the metadata entries matching the given filters
-        for datetime and/or column name.
+        Find the indices for all the metadata entries matching the given
+        filters.
+
         Will return all metadata entry indices if no filters are given.
 
         Parameters
         ----------
-        time : `str` or `~datetime.datetime` optional
-            The string (parsed using the `~sunpy.time.parse_time`) or datetime
-            that you need metadata for.
-
-        colname : `str` optional
+        time : {parse_time_types}, optional
+            A `~sunpy.time.parse_time` parsable string that you need metadata for.
+            Defaults to `None`.
+        colname : `str`, optional
             A string that can be used to narrow results to specific columns.
-
-        indices : `bool` optional
-            If True then return a list of indices, not of MetaDict items.
-            Used when other methods use the filters for selecting metadata entries.
+            Defaults to `None`.
 
         Returns
         -------
-        list : `list`
+        `list`
             A list of integers that contain all matching metadata.
         """
         # Parameters
@@ -183,7 +215,7 @@ class TimeSeriesMetaData:
         # Find all results with suitable timerange.
         results_indices = []
         for i, meta in enumerate(self.metadata):
-            if dt in meta[0] or not(dt):
+            if (not dt) or dt in meta[0]:
                 results_indices.append(i)
 
         # Filter out only those with the correct column.
@@ -194,27 +226,30 @@ class TimeSeriesMetaData:
 
         return results
 
-    def find(self, time=None, colname=None, **kwargs):
+    @add_common_docstring(**_variables_for_parse_time_docstring())
+    def find(self, time=None, colname=None):
         """
-        Find all metadata matching the given filters for datetime and/or column name.
+        Find all metadata matching the given filters.
+
         Will return all metadata entries if no filters are given.
 
         Parameters
         ----------
-        time : `str` or `~datetime.datetime` optional
-            The string (parsed using the `~sunpy.time.parse_time`) or datetime
-            that you need metadata for.
-
-        colname : `str` optional
+        time : {parse_time_types}, optional
+            A `~sunpy.time.parse_time` parsable string that you need metadata for.
+            Defaults to `None`.
+        colname : `str`, optional
             A string that can be used to narrow results to specific columns.
+            Defaults to `None`.
 
         Returns
         -------
-        metadata : `~sunpy.timeseries.metadata.TimeSeriesMetaData`
-            A TimeSeriesMetaData that contain all matching metadata entries.
+        `~sunpy.timeseries.metadata.TimeSeriesMetaData`
+            A `~sunpy.timeseries.metadata.TimeSeriesMetaData` that contain all
+            matching metadata entries.
         """
         # Get the indices
-        indices = self.find_indices(time=time, colname=colname, **kwargs)
+        indices = self.find_indices(time=time, colname=colname)
 
         # Extract the relevant metadata entries
         metadata = []
@@ -235,33 +270,27 @@ class TimeSeriesMetaData:
 
         Returns
         -------
-        metadata : `~sunpy.util.metadata.MetaDict`
+        `~sunpy.util.metadata.MetaDict`
             An ordered Dictionary containing the metadata at the given index.
         """
         return self.metadata[index][2]
 
-    def get(self, keys, time=None, colname=None, **kwargs):
+    @add_common_docstring(**_variables_for_parse_time_docstring())
+    def get(self, keys, time=None, colname=None):
         """
-        Return a TimeSeriesMetaData object of all entries matching the time and
-        colname filters with the dictionaries containing only the key value pairs
-        with the key matching the given input key.
+        Return a `~sunpy.timeseries.metadata.TimeSeriesMetaData` with all
+        entries matching the filters which also contain the given input key.
 
         Parameters
         ----------
         keys : `str`
             The Key/s to be searched in the dictionary.
-
-        time : `str` or `~datetime.datetime` optional
-            The string (parsed using the `~sunpy.time.parse_time`) or datetime
-            that you need metadata for.
-
-        colname : `str` optional
+        time : {parse_time_types}, optional
+            A `~sunpy.time.parse_time` parsable string that you need metadata for.
+            Defaults to `None`.
+        colname : `str`, optional
             A string that can be used to narrow results to specific columns.
 
-        itemised : `bool` optional
-            Option to allow the return of the time ranges and column names
-            (as list) that match each given value.
-
         Returns
         -------
         metadata : `~sunpy.timeseries.metadata.TimeSeriesMetaData`
@@ -270,7 +299,7 @@ class TimeSeriesMetaData:
         """
         # Make a list of keys if only one is given
         if isinstance(keys, str):
-            keys = [ keys ]
+            keys = [keys]
 
         # Find all metadata entries for the given time/colname filters
         full_metadata = self.find(time=time, colname=colname)
@@ -282,16 +311,17 @@ class TimeSeriesMetaData:
             for curkey, value in entry[2].items():
                 for key in keys:
                     if curkey.lower() == key.lower():
-                        metadict.update({key:value})
+                        metadict.update({key: value})
             metadata.append((entry[0], entry[1], metadict))
 
         # Return a TimeSeriesMetaData object
         return TimeSeriesMetaData(meta=metadata)
 
-    def concatenate(self, tsmetadata2, **kwargs):
+    def concatenate(self, tsmetadata2):
         """
-        Combine the metadata from a TimeSeriesMetaData object with the current
-        TimeSeriesMetaData and return as a new TimeSeriesMetaData object.
+        Combine the metadata from a `~sunpy.timeseries.TimeSeriesMetaData` with
+        the current `~sunpy.timeseries.TimeSeriesMetaData` and return as a new
+        `~sunpy.timeseries.TimeSeriesMetaData`.
 
         Parameters
         ----------
@@ -308,29 +338,28 @@ class TimeSeriesMetaData:
 
         return meta
 
-    def update(self, dictionary, time=None, colname=None, row=None, overwrite=False, **kwargs):
+    @add_common_docstring(**_variables_for_parse_time_docstring())
+    def update(self, dictionary, time=None, colname=None, overwrite=False):
         """
-        Make updates to the MetaDict metadata for all matching metadata entries.
+        Updates the `~sunpy.timeseries.TimeSeriesMetaData` for all matching
+        metadata entries.
 
         Parameters
         ----------
-        dictionary : `dict` or `OrderedDict` or `~sunpy.util.metadata.MetaDict`
-            The second TimeSeriesMetaData object.
-
-        time : `str` or `~datetime.datetime` optional
-            The string (parsed using the `~sunpy.time.parse_time`) or datetime
-            to filter the metadata entries updated.
-
-        colname : `str` optional
+        dictionary : `dict`, `OrderedDict`, `~sunpy.util.metadata.MetaDict`
+            The second `~sunpy.timeseries.TimeSeriesMetaData` object.
+        time : {parse_time_types}, optional
+            A `~sunpy.time.parse_time` parsable string that you need metadata for.
+            Defaults to `None`.
+        colname : `str`, optional
             A string that can be used to narrow results to specific columns.
-
-        overwrite : `bool` optional
-            Option to define if the user is able to overwrite already present keys.
-            Defaults to False, designed to stop users from being able to
-            corrupt/damage the metadict values so easily.
+            Defaults to `None`.
+        overwrite : `bool`, optional
+            Allows the user to overwrite already present keys.
+            Defaults to `False`
         """
         # Find all matching metadata entries
-        indices = self.find_indices(time=time, colname=colname, row=row, indices=True)
+        indices = self.find_indices(time=time, colname=colname)
 
         # Now update each matching entries
         for i in indices:
@@ -349,20 +378,21 @@ class TimeSeriesMetaData:
                     self.metadata[i][2][key] = dictionary[key]
 
     def _truncate(self, timerange):
-        """Removes metadata entries outside of the new (truncated) TimeRange.
-        Also adjusts start and end times of time ranges going outside of the
-        truncated time range.
+        """
+        Removes metadata entries outside of the new (truncated)
+        `sunpy.time.TimeRange`. Also adjusts start and end times of time ranges
+        going outside of the truncated time range.
 
         Parameters
         ----------
         timerange : `sunpy.time.TimeRange`
-            Either a time range to truncate to.
+            The time range to truncate to.
         """
         truncated = []
         for metatuple in self.metadata:
             # Get metadata time range parameters
             start = metatuple[0].start
-            end   = metatuple[0].end
+            end = metatuple[0].end
             out_of_range = False
 
             # Find truncations
@@ -388,7 +418,9 @@ class TimeSeriesMetaData:
 
     @property
     def columns(self):
-        """Returns a list of all the names of the columns in the metadata."""
+        """
+        Returns a list of all the names of the columns in the metadata.
+        """
         all_cols = set()
         for metatuple in self.metadata:
             all_cols.update(metatuple[1])
@@ -398,7 +430,10 @@ class TimeSeriesMetaData:
 
     @property
     def metas(self):
-        """Returns a list of all the metadict objects in the TimeSeriesMetaData object."""
+        """
+        Returns a list of all the metadict objects in the
+        `~sunpy.timeseries.TimeSeriesMetaData`.
+        """
         all_metas = []
         for metatuple in self.metadata:
             all_metas.append(metatuple[2])
@@ -406,15 +441,20 @@ class TimeSeriesMetaData:
 
     @property
     def timeranges(self):
-        """Returns a list of all the TimeRange objects the TimeSeriesMetaData object."""
+        """
+        Returns a list of all the `~sunpy.time.TimeRange` in the
+        `~sunpy.timeseries.TimeSeriesMetaData`.
+        """
         all_tr = []
         for metatuple in self.metadata:
             all_tr.append(metatuple[0])
         return all_tr
 
     def values(self):
-        """Returns a list of all the values from the metadict objects in each
-        entry in the TimeSeriesMetaData object."""
+        """
+        Returns a list of all the values from the metadict objects in each
+        entry in the `~sunpy.timeseries.TimeSeriesMetaData`.
+        """
         all_vals = set()
         for metatuple in self.metadata:
             for key, value in metatuple[2].items():
@@ -425,25 +465,29 @@ class TimeSeriesMetaData:
 
     @property
     def time_range(self):
-        """Returns the TimeRange of the entire time series meta data."""
+        """
+        Returns the `~sunpy.time.TimeRange` of the entire timeseries metadata.
+        """
         start = self.metadata[0][0].start
         end = self.metadata[0][0].end
         for metatuple in self.metadata:
             if end < metatuple[0].end:
-               end = metatuple[0].end
+                end = metatuple[0].end
         return TimeRange(start, end)
 
     def _remove_columns(self, colnames):
-        """Removes the given column/s from the TimeSeriesMetaData object.
+        """
+        Removes the given column(s) from the
+        `~sunpy.timeseries.TimeSeriesMetaData`.
 
         Parameters
         ----------
-        colnames : `str` or `list`
-            The name or names of the columns to be removed.
+        colnames : `str`, `list` of `str`
+            The name(s) of the columns to be removed.
         """
         # Parameters
         if isinstance(colnames, str):
-            colnames = [ colnames ]
+            colnames = [colnames]
 
         # Create a new list with all metadata entries without colnames
         reduced = []
@@ -460,17 +504,14 @@ class TimeSeriesMetaData:
         # Update the original list
         self.metadata = reduced
 
-
-
     def _rename_column(self, old, new):
         """
-        Change the name of a column in all the metadata entries.
+        Change the name of a column in the metadata entries.
 
         Parameters
         ----------
         old : `str`
             The original column name to be changed.
-
         new : `str`
             The new column name.
         """
@@ -480,11 +521,11 @@ class TimeSeriesMetaData:
             colnames = [w.replace(old, new) for w in colnames]
 
             # Replace values
-            self.metadata[i] = ( self.metadata[i][0], colnames, self.metadata[i][2] )
+            self.metadata[i] = (self.metadata[i][0], colnames, self.metadata[i][2])
 
     def _validate_meta(self, meta):
         """
-        Validate a meta argument.
+        Validate a metadata argument.
         """
         # Checking for metadata that may overlap.
         indices = range(0, len(self.metadata))
@@ -495,31 +536,33 @@ class TimeSeriesMetaData:
                 col_overlap = list(set(self.metadata[i][1]) & set(self.metadata[j][1]))
                 # If we have an overlap then show a warning
                 if col_overlap:
-                    warnings.warn_explicit('Metadata entries ' + str(i) + ' and ' + str(j) + ' contain interleaved data.',
-                                           Warning, __file__, inspect.currentframe().f_back.f_lineno)
-
-        # ToDo: Check all entries are in tr.start time order.
+                    warnings.warn(f'Metadata entries {i} and {j} contain interleaved data.', SunpyUserWarning)
 
+        # TODO: Check all entries are in tr.start time order.
         return True
 
     def to_string(self, depth=10, width=99):
         """
-        Print a table-like representation of the TimeSeriesMetaData object.
+        Print a table-like representation of the
+        `~sunpy.timeseries.TimeSeriesMetaData`.
 
         Parameters
         ----------
-        depth : `int`
-            The maximum number of lines to show for each entry. Metadata
-            dictionaries and column lists will be truncated if this is small.
-
-        width : `int`
-            The number of characters wide to make the entire table.
+        depth : `int`, optional
+            The maximum number of lines to show for each entry.
+            Metadata dictionaries and column lists will be truncated if this is small.
+            Defaults to 10.
+        width : `int`, optional
+            The number of characters wide to make the entire table. Defaults to 99.
         """
         # Parameters
         colspace = ' | '
         liswidths = (26, 15, width-2-2*len(colspace) - 26 - 15)
-        colheadings = '|' + 'TimeRange'.ljust(100)[:liswidths[0]] + colspace + 'Columns'.ljust(100)[:liswidths[1]] + colspace + 'Meta'.ljust(100)[:liswidths[2]]  + '|'
-        rowspace = "-" * (liswidths[0] + len(colspace) + liswidths[1] + len(colspace) + liswidths[2])
+        colheadings = '|' + 'TimeRange'.ljust(100)[:liswidths[0]] + colspace
+        colheadings += 'Columns'.ljust(100)[:liswidths[1]] + colspace
+        colheadings += 'Meta'.ljust(100)[:liswidths[2]] + '|'
+        rowspace = "-" * (liswidths[0] + len(colspace) + liswidths[1]
+                          + len(colspace) + liswidths[2])
         rowspace = '|' + rowspace + '|'
 
         # Headings
@@ -529,10 +572,10 @@ class TimeSeriesMetaData:
         for entry in self.metadata:
             # Make lists for each of the columns for each metadata entry
             # Padded to the widths given in liswidths
-            lis_range = [ str(entry[0].start), '            to            ', str(entry[0].end) ]
+            lis_range = [str(entry[0].start), '            to            ', str(entry[0].end)]
             # Shorten TimeRange representation if depth of only 2
             if depth == 2:
-                lis_range = [ str(entry[0].start), str(entry[0].end) ]
+                lis_range = [str(entry[0].start), str(entry[0].end)]
             liscols = []
             for col in entry[1]:
                 liscols.append(col.ljust(100)[:liswidths[1]])
@@ -545,7 +588,7 @@ class TimeSeriesMetaData:
             for i in range(0, depth):
                 # What to do in the event any of the lists have more entries
                 # then the current depth
-                if len(lis_range) > i or len(entry[1]) > i or len(lismeta) > i :
+                if len(lis_range) > i or len(entry[1]) > i or len(lismeta) > i:
                     # The start of the line Str is just a vertical bar/pipe
                     line = '|'
                     # Check we have a time range entry to print
@@ -612,5 +655,6 @@ class TimeSeriesMetaData:
 
     def __repr__(self):
         return self.to_string()
+
     def __str__(self):
-        return self.to_string()
\ No newline at end of file
+        return self.to_string()
